///
/// Copyright (c) 2015, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

library haystack.huri;

import 'HVal.dart';
import 'str-utils.dart';

/// HUri models a URI as a string.
///
/// See [Project Haystack](http://project-haystack.org/doc/TagModel#tagKinds)
class HUri extends HVal {

  /** Singleton value for empty URI */
  static final HUri EMPTY = new HUri._("");

  /** Factory constructor from string value */
  factory HUri (String val) {
    if (val == null || val.isEmpty) return EMPTY;
    return new HUri._(val);
  }

  /** Private constructor */
  HUri._(this.val);

  /** String value of URI */
  final String val;

  /** Hash code is based on string value */
  int get hashCode => val.hashCode;

  /** Equals is based on string value */
  bool operator ==(Object that) => that is HUri && this.val == that.val;

  /** Return value string. */
  @override
  String toString() => val;

  /** Encode as "u:<val>" */
  String toJson() => "u:$val";

  /** Encode using "`" back ticks */
  String toZinc() {
    StringBuffer s = new StringBuffer();
    s.write('`');

    val.codeUnits.forEach((int c) {
      if (c < SPACE_CHAR) throw new FormatException("Invalid URI char: '${new String.fromCharCode(c)}'", val);
      if (c == BACKQUOTE_CHAR) s.write('\\');
      s.writeCharCode(c);
    });

    s.writeCharCode(BACKQUOTE_CHAR);
    return s.toString();
  }
}
